var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { LitElement, html, customElement, css, property, } from 'lit-element';
import { unsafeHTML } from 'lit-html/directives/unsafe-html';
import { until } from 'lit-html/directives/until';
import 'prismjs';
import 'prismjs/components/prism-js-templates.js';
import { EMPTY_INDEX } from './constants.js';
import { fetchProject, fetchTemplate, importJs } from './util.js';
import './demo-snippet-layout.js';
const { highlight, languages } = window.Prism;
import prismTheme from './prism-theme.js';
let DemoSnippet = class DemoSnippet extends LitElement {
    constructor() {
        super(...arguments);
        this.initialized = false;
        this.imported = false;
        this.htmlReady = false;
        this.snippetsReady = Promise.resolve([EMPTY_INDEX]);
        this.templateReady = Promise.resolve('');
        this.jsImported = Promise.resolve(() => { });
    }
    async renderSnippets(projectFetched) {
        const fileRecords = await projectFetched;
        let firstEditor = true;
        let index = 0;
        // Sort so HTML comes first
        const records = fileRecords.sort((a, b) => {
            return a.extension < b.extension ? -1 : 1;
        });
        const tabs = records.map(fileRecord => {
            let grammar;
            const { extension } = fileRecord;
            switch (extension) {
                case 'js':
                    grammar = languages.javascript;
                    break;
                default:
                    grammar = languages.markup;
            }
            let content = fileRecord.content;
            if (extension === 'js') {
                content = content.replace(/'(.+)(?=@vaadin)/g, `'`);
                if (this.whenDefined) {
                    content = content.replace('export default (document => {', `customElements.whenDefined('${this.whenDefined}').then(() => {`);
                }
            }
            const formatted = unsafeHTML(highlight(content, grammar, extension));
            const tResult = html `
        <span
          slot="tab"
          class=${'link-' + index.toString()}
          ?selected=${firstEditor}
        >
          ${extension}
        </span>
        <pre
          slot="code"
          class=${'link-' + index.toString()}
          ?selected=${firstEditor}
          ><code>${formatted}</code></pre>
      `;
            firstEditor = false;
            index++;
            return tResult;
        });
        return tabs;
    }
    static get styles() {
        return [
            prismTheme,
            css `
        :host {
          display: block;
          max-width: 800px;
        }

        #output {
          border: solid 1px #ccc;
          padding: 1rem;
        }

        pre {
          margin: 0;
        }
      `
        ];
    }
    async renderTemplate(templateFetched) {
        const template = await templateFetched;
        return html `
      <div id="output">
        ${unsafeHTML(template)}
      </div>
    `;
    }
    render() {
        const isNewProject = this.projectPath &&
            this.templatePath &&
            this.lastProjectPath !== this.projectPath;
        if (isNewProject) {
            this.lastProjectPath = this.projectPath;
            this.snippetsReady = fetchProject(this.projectPath);
            this.templateReady = fetchTemplate(this.templatePath).then(tpl => {
                this.htmlReady = true;
                return tpl;
            });
            if (this.jsPath) {
                this.jsImported = importJs(this.jsPath).then(cb => {
                    this.imported = true;
                    return cb;
                });
            }
        }
        return html `
      <div id="wrapper">
        <demo-snippet-layout>
          ${until(this.renderSnippets(this.snippetsReady))}
        </demo-snippet-layout>
        ${until(this.renderTemplate(this.templateReady))}
      </div>
    `;
    }
    async updated(props) {
        super.updated(props);
        if ((props.has('imported') || props.has('htmlReady')) && this.htmlReady && this.imported) {
            const cb = await this.jsImported;
            cb(this.shadowRoot.querySelector('#output'));
        }
    }
};
__decorate([
    property({ attribute: 'template-path', type: String })
], DemoSnippet.prototype, "templatePath", void 0);
__decorate([
    property({ attribute: 'project-path', type: String })
], DemoSnippet.prototype, "projectPath", void 0);
__decorate([
    property({ attribute: 'js-path', type: String })
], DemoSnippet.prototype, "jsPath", void 0);
__decorate([
    property({ attribute: 'when-defined', type: String })
], DemoSnippet.prototype, "whenDefined", void 0);
__decorate([
    property({ type: Boolean })
], DemoSnippet.prototype, "initialized", void 0);
__decorate([
    property({ type: Boolean })
], DemoSnippet.prototype, "imported", void 0);
__decorate([
    property({ type: Boolean })
], DemoSnippet.prototype, "htmlReady", void 0);
DemoSnippet = __decorate([
    customElement('demo-snippet')
], DemoSnippet);
export { DemoSnippet };
